@@ -611,6 +611,8 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 			xmlChar *query;
 			xmlXPathObjectPtr newlist;
 			xmlNodePtr parent = node->parent;
+			xmlDocPtr olddoc;
+			xmlNodePtr oldnode;
 
 			if (comp->pattern[0] == '/')
 			    query = xmlStrdup(comp->pattern);
@@ -618,7 +620,13 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 			    query = xmlStrdup((const xmlChar *)"//");
 			    query = xmlStrcat(query, comp->pattern);
 			}
+			oldnode = ctxt->xpathCtxt->node;
+			olddoc = ctxt->xpathCtxt->doc;
+			ctxt->xpathCtxt->node = node;
+			ctxt->xpathCtxt->doc = doc;
 			newlist = xmlXPathEval(query, ctxt->xpathCtxt);
+			ctxt->xpathCtxt->node = oldnode;
+			ctxt->xpathCtxt->doc = olddoc;
 			xmlFree(query);
 			if (newlist == NULL)
 			    return(-1);
@@ -630,11 +638,10 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 			if ((parent == NULL) || (node->doc == NULL))
 			    nocache = 1;
 			else {
-			    while (parent->parent != NULL)
-				parent = parent->parent;
-			    if (((parent->type != XML_DOCUMENT_NODE) &&
-				 (parent->type != XML_HTML_DOCUMENT_NODE)) ||
-				 (parent != (xmlNodePtr) node->doc))
+			    if ((doc->name != NULL) &&
+				(doc->name[0] == ' ') &&
+				(xmlStrEqual(BAD_CAST doc->name, 
+					     BAD_CAST " fake node libxslt")))
 				nocache = 1;
 			}
 			if (nocache == 0) {
@@ -2162,6 +2169,19 @@ xsltGetTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
 		}
 		list = list->next;
 	    }
+	} else if ((node->type == XML_PI_NODE) ||
+		   (node->type == XML_COMMENT_NODE)) {
+	    list = curstyle->elemMatch;
+	    while ((list != NULL) &&
+		   ((ret == NULL)  || (list->priority > priority))) {
+		if (xsltTestCompMatch(ctxt, list, node,
+				      ctxt->mode, ctxt->modeURI)) {
+		    ret = list->template;
+		    priority = list->priority;
+		    break;
+		}
+		list = list->next;
+	    }
 	}
 
 	if (node->_private != NULL) {
